@@ -3,9 +3,9 @@
     # Provide path to package dylib for LuaJIT FFI
     .Call(`_luajr_locate_dylib`, getLoadedDLLs()[["luajr"]][["path"]])
     # Provide path to luajr module
-    .Call(`_luajr_locate_module`, system.file("module", "luajr.lua", package = "luajr"))
+    .Call(`_luajr_locate_module`, system.file("Lua", "luajr.lua", package = "luajr"))
     # Provide path to debugger.lua
-    .Call(`_luajr_locate_debugger`, system.file("module", "debugger.lua", package = "luajr"))
+    .Call(`_luajr_locate_debugger`, system.file("Lua", "debugger.lua", package = "luajr"))
     invisible()
 }
 
